# (c) cavaliba.com - data - schema.py

import copy
import yaml
import sys

import app_home.cache as cache

from app_data.models import DataClass
from app_data.models import DataSchema


# New V3.19

class Schema:

    def __init__(self):

        self.classname = None
        self.displayname = None
        self.handle_method = None
        self.is_enabled = None
        self.icon = None

        self.p_admin = None
        self.p_create = None
        self.p_read = None
        self.p_update = None
        self.p_delete = None
        self.p_import = None
        self.p_export = None

        self.options = {}
        # keyname_mode: edit(*) | auto
        # handle_option
        # displayname_option

        #self.order = None
        #self.page = None

        # section

    def __str__(self):
        return f"{self.classname}"
    

        
    @classmethod
    def from_name(cls, classname=None):
        ''' load from DB or None'''

        if not classname:
            return
        
        if classname in cache.cache_schema2:
            return copy.deepcopy(cache.cache_schema2[classname])
            
        obj = DataClass.objects.filter(keyname=classname).first()
        if not obj:
            cache.cache_schema2.pop(classname, None)
            return


        schema = cls()
        schema.classname = classname
        schema.displayname = obj.displayname
        schema.handle_method = obj.handle_method
        schema.is_enabled = obj.is_enabled
        schema.icon = obj.icon

        schema.p_admin = obj.p_admin
        schema.p_create = obj.p_create
        schema.p_read = obj.p_read
        schema.p_update = obj.p_update
        schema.p_delete = obj.p_delete
        schema.p_import = obj.p_import
        schema.p_export = obj.p_export

        # options
        try:
            schema.options = yaml.safe_load(obj.options)
            if type(schema.options) is not dict:
                schema.options = {}
        except:
            schema.options = {}

        # set default options if missing
        schema.options['keyname_mode'] = schema.get_keyname_mode()




        # cache update
        cache.cache_schema2[classname] = copy.deepcopy(schema)

        return schema
    



    def get_keyname_mode(self):

        VALID = ['edit', 'auto']
        DEFAULT = 'edit'

        v = DEFAULT
        if self.options:
            v = self.options.get('keyname_mode', DEFAULT)
            if v not in VALID:
                v = DEFAULT
        return v  
